==> 1.3.0.zip: books-without-borders/src/main/java/net/knarcraft/bookswithoutborders/BooksWithoutBorders.java <==
package net.knarcraft.bookswithoutborders;

import net.knarcraft.bookswithoutborders.command.CommandBooksWithoutBorders;
import net.knarcraft.bookswithoutborders.command.CommandCopy;
import net.knarcraft.bookswithoutborders.command.CommandDecrypt;
import net.knarcraft.bookswithoutborders.command.CommandDelete;
import net.knarcraft.bookswithoutborders.command.CommandDeletePublic;
import net.knarcraft.bookswithoutborders.command.CommandEncrypt;
import net.knarcraft.bookswithoutborders.command.CommandFormat;
import net.knarcraft.bookswithoutborders.command.CommandGive;
import net.knarcraft.bookswithoutborders.command.CommandGivePublic;
import net.knarcraft.bookswithoutborders.command.CommandGroupEncrypt;
import net.knarcraft.bookswithoutborders.command.CommandLoad;
import net.knarcraft.bookswithoutborders.command.CommandLoadPublic;
import net.knarcraft.bookswithoutborders.command.CommandReload;
import net.knarcraft.bookswithoutborders.command.CommandSave;
import net.knarcraft.bookswithoutborders.command.CommandSavePublic;
import net.knarcraft.bookswithoutborders.command.CommandSetAuthor;
import net.knarcraft.bookswithoutborders.command.CommandSetBookPrice;
import net.knarcraft.bookswithoutborders.command.CommandSetGeneration;
import net.knarcraft.bookswithoutborders.command.CommandSetLore;
import net.knarcraft.bookswithoutborders.command.CommandSetTitle;
import net.knarcraft.bookswithoutborders.command.CommandUnSign;
import net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig;
import net.knarcraft.bookswithoutborders.listener.BookEventListener;
import net.knarcraft.bookswithoutborders.listener.PlayerEventListener;
import net.knarcraft.bookswithoutborders.listener.SignEventListener;
import net.knarcraft.bookswithoutborders.utility.FileHelper;
import net.knarcraft.bookswithoutborders.utility.UpdateChecker;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.command.ConsoleCommandSender;
import org.bukkit.command.PluginCommand;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemFactory;
import org.bukkit.plugin.PluginDescriptionFile;
import org.bukkit.plugin.PluginManager;
import org.bukkit.plugin.java.JavaPlugin;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import static net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig.getBookFolder;
import static net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig.getErrorColor;
import static net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig.getSlash;
import static net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig.getSuccessColor;

/**
 * The main Books Without Borders class
 */
public class BooksWithoutBorders extends JavaPlugin {

    private static ItemFactory itemFactory;
    private static Map<UUID, List<String>> playerBooksList;
    private static List<String> publicBooksList;
    private static BooksWithoutBorders booksWithoutBorders;
    private static ConsoleCommandSender consoleSender;

    /**
     * Gets the console sender for printing to the console
     *
     * @return <p>The console's console sender</p>
     */
    public static ConsoleCommandSender getConsoleSender() {
        return consoleSender;
    }

    /**
     * Gets available books
     *
     * @param sender    <p>The sender wanting to see available books</p>
     * @param getPublic <p>Whether to get available public books</p>
     * @return <p>A list of available books</p>
     */
    public static List<String> getAvailableBooks(CommandSender sender, boolean getPublic) {
        if (getPublic) {
            return new ArrayList<>(publicBooksList);
        } else if (sender instanceof Player player) {
            UUID playerUUID = player.getUniqueId();
            if (!playerBooksList.containsKey(playerUUID)) {
                List<String> newFiles = FileHelper.listFiles(sender, false);
                playerBooksList.put(playerUUID, newFiles);
            }
            return playerBooksList.get(playerUUID);
        } else {
            return new ArrayList<>();
        }
    }

    /**
     * Updates available books
     *
     * @param sender       <p>The sender to update books for</p>
     * @param updatePublic <p>Whether to update public books</p>
     */
    public static void updateBooks(CommandSender sender, boolean updatePublic) {
        List<String> newFiles = FileHelper.listFiles(sender, updatePublic);
        if (updatePublic) {
            publicBooksList = newFiles;
        } else if (sender instanceof Player player) {
            playerBooksList.put(player.getUniqueId(), newFiles);
        }
    }

    @Override
    public void onEnable() {
        FileConfiguration config = this.getConfig();
        config.options().copyDefaults(true);
        this.saveDefaultConfig();

        //Get plugin info
        PluginDescriptionFile pluginDescriptionFile = this.getDescription();
        String pluginVersion = pluginDescriptionFile.getVersion();

        booksWithoutBorders = this;
        consoleSender = this.getServer().getConsoleSender();
        playerBooksList = new HashMap<>();
        BooksWithoutBordersConfig.initialize(this);
        publicBooksList = FileHelper.listFiles(consoleSender, true);

        PluginManager pluginManager = this.getServer().getPluginManager();

        if (getSlash() != null && initialize()) {
            pluginManager.registerEvents(new PlayerEventListener(), this);
            pluginManager.registerEvents(new SignEventListener(), this);
            pluginManager.registerEvents(new BookEventListener(), this);
        } else {
            this.getPluginLoader().disablePlugin(this);
        }

        registerCommands();

        UpdateChecker.checkForUpdate(this, "https://api.spigotmc.org/legacy/update.php?resource=96069",
                () -> pluginVersion, null);
    }

    /**
     * Gets an instance of this plugin
     *
     * @return <p>An instance of this plugin</p>
     */
    public static BooksWithoutBorders getInstance() {
        return booksWithoutBorders;
    }

    /**
     * Registers all commands used by this plugin
     */
    private void registerCommands() {
        registerCommand("giveBook", new CommandGive());
        registerCommand("givePublicBook", new CommandGivePublic());
        registerCommand("decryptBook", new CommandDecrypt());
        registerCommand("groupEncryptBook", new CommandGroupEncrypt());
        registerCommand("deleteBook", new CommandDelete());
        registerCommand("deletePublicBook", new CommandDeletePublic());
        registerCommand("copyBook", new CommandCopy());
        registerCommand("unSignBook", new CommandUnSign());
        registerCommand("encryptBook", new CommandEncrypt());
        registerCommand("setBookPrice", new CommandSetBookPrice());
        registerCommand("setLore", new CommandSetLore());
        registerCommand("savePublicBook", new CommandSavePublic());
        registerCommand("saveBook", new CommandSave());
        registerCommand("setBookAuthor", new CommandSetAuthor());
        registerCommand("setTitle", new CommandSetTitle());
        registerCommand("loadBook", new CommandLoad());
        registerCommand("loadPublicBook", new CommandLoadPublic());
        registerCommand("booksWithoutBorders", new CommandBooksWithoutBorders());
        registerCommand("reload", new CommandReload());
        registerCommand("formatBook", new CommandFormat());
        registerCommand("setBookGeneration", new CommandSetGeneration());
    }

    /**
     * Registers a command
     *
     * @param commandName <p>The name of the command to register</p>
     * @param executor    <p>The executor to register for the command</p>
     */
    private void registerCommand(String commandName, CommandExecutor executor) {
        PluginCommand pluginCommand = this.getCommand(commandName);
        if (pluginCommand != null) {
            pluginCommand.setExecutor(executor);
        } else {
            sendErrorMessage(consoleSender, "Failed to register command " + commandName);
        }
    }

    /**
     * Initializes the plugin, loading and fixing the config file
     *
     * @return <p>True if successful</p>
     */
    private boolean initialize() {
        //Initialize Item Factory
        try {
            itemFactory = this.getServer().getItemFactory();
        } catch (java.lang.NoSuchMethodError nsmE) {
            sendErrorMessage(consoleSender, "Warning! [BooksWithoutBorders] failed to initialize!");
            sendErrorMessage(consoleSender, "Please confirm the correct version of [BooksWithoutBorders] is");
            sendErrorMessage(consoleSender, "being run for this version of bukkit!");
            return false;
        }

        //Load config
        if (!BooksWithoutBordersConfig.loadConfig()) {
            return false;
        }

        //Save config with loaded values to fix invalid config values
        BooksWithoutBordersConfig.saveConfigValues();

        return testFileSaving();
    }

    /**
     * Gets the server's item factory
     *
     * @return <p>The server's item factory</p>
     */
    public static ItemFactory getItemFactory() {
        return itemFactory;
    }

    /**
     * Makes sure necessary folders exist
     *
     * @return <p>True if necessary folders exist</p>
     */
    private boolean testFileSaving() {
        File fileTest = new File(getBookFolder());
        File encryptedFileTest = new File(getBookFolder() + "Encrypted" + getSlash());
        if (!fileTest.exists()) {
            try {
                if (!fileTest.mkdir()) {
                    sendErrorMessage(consoleSender, "Saving failed! Aborting...");
                    return false;
                }
            } catch (Exception e) {
                e.printStackTrace();
                return false;
            }
        }
        if (!encryptedFileTest.exists()) {
            try {
                if (!encryptedFileTest.mkdir()) {
                    sendErrorMessage(consoleSender, "Saving failed! Aborting...");
                    return false;
                }
            } catch (Exception e) {
                e.printStackTrace();
                return false;
            }
        }
        return true;
    }

    /**
     * Sends a success message to a command sender (player or a console)
     *
     * @param sender  <p>The sender to send the message to</p>
     * @param message <p>The message to send</p>
     */
    public static void sendSuccessMessage(CommandSender sender, String message) {
        sender.sendMessage(getSuccessColor() + message);
    }

    /**
     * Sends an error message to a command sender (player or a console)
     *
     * @param sender  <p>The sender to send the message to</p>
     * @param message <p>The message to send</p>
     */
    public static void sendErrorMessage(CommandSender sender, String message) {
        sender.sendMessage(getErrorColor() + message);
    }

}

==> 1.3.5.zip: books-without-borders/src/main/java/net/knarcraft/bookswithoutborders/BooksWithoutBorders.java <==
package net.knarcraft.bookswithoutborders;

import net.knarcraft.bookswithoutborders.command.CommandBooksWithoutBorders;
import net.knarcraft.bookswithoutborders.command.CommandClear;
import net.knarcraft.bookswithoutborders.command.CommandCopy;
import net.knarcraft.bookswithoutborders.command.CommandDecrypt;
import net.knarcraft.bookswithoutborders.command.CommandDelete;
import net.knarcraft.bookswithoutborders.command.CommandDeletePublic;
import net.knarcraft.bookswithoutborders.command.CommandEncrypt;
import net.knarcraft.bookswithoutborders.command.CommandFormat;
import net.knarcraft.bookswithoutborders.command.CommandGive;
import net.knarcraft.bookswithoutborders.command.CommandGivePublic;
import net.knarcraft.bookswithoutborders.command.CommandGroupEncrypt;
import net.knarcraft.bookswithoutborders.command.CommandLoad;
import net.knarcraft.bookswithoutborders.command.CommandLoadPublic;
import net.knarcraft.bookswithoutborders.command.CommandReload;
import net.knarcraft.bookswithoutborders.command.CommandSave;
import net.knarcraft.bookswithoutborders.command.CommandSavePublic;
import net.knarcraft.bookswithoutborders.command.CommandSetAuthor;
import net.knarcraft.bookswithoutborders.command.CommandSetBookPrice;
import net.knarcraft.bookswithoutborders.command.CommandSetGeneration;
import net.knarcraft.bookswithoutborders.command.CommandSetLore;
import net.knarcraft.bookswithoutborders.command.CommandSetTitle;
import net.knarcraft.bookswithoutborders.command.CommandUnSign;
import net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig;
import net.knarcraft.bookswithoutborders.listener.BookEventListener;
import net.knarcraft.bookswithoutborders.listener.BookshelfListener;
import net.knarcraft.bookswithoutborders.listener.PlayerEventListener;
import net.knarcraft.bookswithoutborders.listener.SignEventListener;
import net.knarcraft.bookswithoutborders.utility.BookFileHelper;
import net.knarcraft.knarlib.util.UpdateChecker;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.command.ConsoleCommandSender;
import org.bukkit.command.PluginCommand;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemFactory;
import org.bukkit.plugin.PluginDescriptionFile;
import org.bukkit.plugin.PluginManager;
import org.bukkit.plugin.java.JavaPlugin;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.logging.Level;

import static net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig.getBookFolder;
import static net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig.getErrorColor;
import static net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig.getSlash;
import static net.knarcraft.bookswithoutborders.config.BooksWithoutBordersConfig.getSuccessColor;

/**
 * The main Books Without Borders class
 */
public class BooksWithoutBorders extends JavaPlugin {

    private static ItemFactory itemFactory;
    private static Map<UUID, List<String>> playerBooksList;
    private static List<String> publicBooksList;
    private static BooksWithoutBorders booksWithoutBorders;
    private static ConsoleCommandSender consoleSender;

    /**
     * Gets the console sender for printing to the console
     *
     * @return <p>The console's console sender</p>
     */
    public static ConsoleCommandSender getConsoleSender() {
        return consoleSender;
    }

    /**
     * Gets available books
     *
     * @param sender    <p>The sender wanting to see available books</p>
     * @param getPublic <p>Whether to get available public books</p>
     * @return <p>A list of available books</p>
     */
    public static List<String> getAvailableBooks(CommandSender sender, boolean getPublic) {
        if (getPublic) {
            return new ArrayList<>(publicBooksList);
        } else if (sender instanceof Player player) {
            UUID playerUUID = player.getUniqueId();
            if (!playerBooksList.containsKey(playerUUID)) {
                List<String> newFiles = BookFileHelper.listFiles(sender, false);
                playerBooksList.put(playerUUID, newFiles);
            }
            return playerBooksList.get(playerUUID);
        } else {
            return new ArrayList<>();
        }
    }

    /**
     * Updates available books
     *
     * @param sender       <p>The sender to update books for</p>
     * @param updatePublic <p>Whether to update public books</p>
     */
    public static void updateBooks(CommandSender sender, boolean updatePublic) {
        List<String> newFiles = BookFileHelper.listFiles(sender, updatePublic);
        if (updatePublic) {
            publicBooksList = newFiles;
        } else if (sender instanceof Player player) {
            playerBooksList.put(player.getUniqueId(), newFiles);
        }
    }

    @Override
    public void onEnable() {
        FileConfiguration config = this.getConfig();
        config.options().copyDefaults(true);
        this.saveDefaultConfig();

        //Get plugin info
        PluginDescriptionFile pluginDescriptionFile = this.getDescription();
        String pluginVersion = pluginDescriptionFile.getVersion();

        booksWithoutBorders = this;
        consoleSender = this.getServer().getConsoleSender();
        playerBooksList = new HashMap<>();
        BooksWithoutBordersConfig.initialize(this);
        publicBooksList = BookFileHelper.listFiles(consoleSender, true);

        PluginManager pluginManager = this.getServer().getPluginManager();

        if (getSlash() != null && initialize()) {
            pluginManager.registerEvents(new PlayerEventListener(), this);
            pluginManager.registerEvents(new SignEventListener(), this);
            pluginManager.registerEvents(new BookEventListener(), this);
            pluginManager.registerEvents(new BookshelfListener(), this);
        } else {
            this.getPluginLoader().disablePlugin(this);
        }

        registerCommands();

        UpdateChecker.checkForUpdate(this, "https://api.spigotmc.org/legacy/update.php?resource=96069",
                () -> pluginVersion, null);
    }

    /**
     * Gets an instance of this plugin
     *
     * @return <p>An instance of this plugin</p>
     */
    public static BooksWithoutBorders getInstance() {
        return booksWithoutBorders;
    }

    /**
     * Registers all commands used by this plugin
     */
    private void registerCommands() {
        registerCommand("giveBook", new CommandGive());
        registerCommand("givePublicBook", new CommandGivePublic());
        registerCommand("decryptBook", new CommandDecrypt());
        registerCommand("groupEncryptBook", new CommandGroupEncrypt());
        registerCommand("deleteBook", new CommandDelete());
        registerCommand("deletePublicBook", new CommandDeletePublic());
        registerCommand("copyBook", new CommandCopy());
        registerCommand("unSignBook", new CommandUnSign());
        registerCommand("encryptBook", new CommandEncrypt());
        registerCommand("setBookPrice", new CommandSetBookPrice());
        registerCommand("setLore", new CommandSetLore());
        registerCommand("savePublicBook", new CommandSavePublic());
        registerCommand("saveBook", new CommandSave());
        registerCommand("setBookAuthor", new CommandSetAuthor());
        registerCommand("setTitle", new CommandSetTitle());
        registerCommand("loadBook", new CommandLoad());
        registerCommand("loadPublicBook", new CommandLoadPublic());
        registerCommand("booksWithoutBorders", new CommandBooksWithoutBorders());
        registerCommand("reload", new CommandReload());
        registerCommand("formatBook", new CommandFormat());
        registerCommand("setBookGeneration", new CommandSetGeneration());
        registerCommand("clearBook", new CommandClear());
    }

    /**
     * Registers a command
     *
     * @param commandName <p>The name of the command to register</p>
     * @param executor    <p>The executor to register for the command</p>
     */
    private void registerCommand(String commandName, CommandExecutor executor) {
        PluginCommand pluginCommand = this.getCommand(commandName);
        if (pluginCommand != null) {
            pluginCommand.setExecutor(executor);
        } else {
            sendErrorMessage(consoleSender, "Failed to register command " + commandName);
        }
    }

    /**
     * Initializes the plugin, loading and fixing the config file
     *
     * @return <p>True if successful</p>
     */
    private boolean initialize() {
        //Initialize Item Factory
        try {
            itemFactory = this.getServer().getItemFactory();
        } catch (java.lang.NoSuchMethodError nsmE) {
            sendErrorMessage(consoleSender, "Warning! [BooksWithoutBorders] failed to initialize!");
            sendErrorMessage(consoleSender, "Please confirm the correct version of [BooksWithoutBorders] is");
            sendErrorMessage(consoleSender, "being run for this version of bukkit!");
            return false;
        }

        //Load config
        if (!BooksWithoutBordersConfig.loadConfig()) {
            return false;
        }

        //Save config with loaded values to fix invalid config values
        BooksWithoutBordersConfig.saveConfigValues();

        return testFileSaving();
    }

    /**
     * Gets the server's item factory
     *
     * @return <p>The server's item factory</p>
     */
    public static ItemFactory getItemFactory() {
        return itemFactory;
    }

    /**
     * Makes sure necessary folders exist
     *
     * @return <p>True if necessary folders exist</p>
     */
    private boolean testFileSaving() {
        File fileTest = new File(getBookFolder());
        File encryptedFileTest = new File(getBookFolder() + "Encrypted" + getSlash());
        if (!fileTest.exists()) {
            try {
                if (!fileTest.mkdir()) {
                    sendErrorMessage(consoleSender, "Saving failed! Aborting...");
                    return false;
                }
            } catch (Exception exception) {
                BooksWithoutBorders.getInstance().getLogger().log(Level.SEVERE, "Unable to create necessary folders");
                return false;
            }
        }
        if (!encryptedFileTest.exists()) {
            try {
                if (!encryptedFileTest.mkdir()) {
                    sendErrorMessage(consoleSender, "Saving failed! Aborting...");
                    return false;
                }
            } catch (Exception exception) {
                BooksWithoutBorders.getInstance().getLogger().log(Level.SEVERE, "Unable to create necessary folders");
                return false;
            }
        }
        return true;
    }

    /**
     * Sends a success message to a command sender (player or a console)
     *
     * @param sender  <p>The sender to send the message to</p>
     * @param message <p>The message to send</p>
     */
    public static void sendSuccessMessage(CommandSender sender, String message) {
        sender.sendMessage(getSuccessColor() + message);
    }

    /**
     * Sends an error message to a command sender (player or a console)
     *
     * @param sender  <p>The sender to send the message to</p>
     * @param message <p>The message to send</p>
     */
    public static void sendErrorMessage(CommandSender sender, String message) {
        sender.sendMessage(getErrorColor() + message);
    }

}
